@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+# pep8-ignore: E501
 
 '''
 ====================================
@@ -37,9 +38,7 @@ I wish to thank the following people for their input:
 
 '''
 
-__version__ = '0.6'
-
-import socket
+__version__ = '1.0.0'
 
 
 try:
@@ -49,20 +48,20 @@ except NameError:
         '''
         Stringifies an int or long in base 2.
         '''
-        if x < 0: 
+        if x < 0:
             return '-' + bin(-x)
         out = []
-        if x == 0: 
+        if x == 0:
             out.append('0')
         while x > 0:
             out.append('01'[x & 1])
             x >>= 1
             pass
-        try: 
+        try:
             return '0b' + ''.join(reversed(out))
-        except NameError, ne2: 
+        except NameError:
             out.reverse()
-        return '0b' + ''.join(out) 
+        return '0b' + ''.join(out)
 
 
 class IP(object):
@@ -70,7 +69,7 @@ class IP(object):
     Represents a single IP address.
 
     :param ip: the ip address
-    :type ip: :class:`IP` or str or long or int 
+    :type ip: :class:`IP` or str or long or int
 
     >>> localhost = IP("127.0.0.1")
     >>> print localhost
@@ -86,64 +85,64 @@ class IP(object):
         '4': '0100', '5': '0101', '6': '0110', '7': '0111',
         '8': '1000', '9': '1001', 'a': '1010', 'b': '1011',
         'c': '1100', 'd': '1101', 'e': '1110', 'f': '1111'
-        }
+    }
 
     # IP range specific information, see IANA allocations.
     _range = {
         4: {
-            '01'                    : 'CLASS A',
-            '10'                    : 'CLASS B',
-            '110'                   : 'CLASS C',
-            '1110'                  : 'CLASS D MULTICAST',
-            '11100000'              : 'CLASS D LINKLOCAL',
-            '1111'                  : 'CLASS E',
-            '00001010'              : 'PRIVATE RFC1918', # 10/8
-            '101011000001'          : 'PRIVATE RFC1918', # 172.16/12
-            '1100000010101000'      : 'PRIVATE RFC1918', # 192.168/16
-            },
+            '01':                     'CLASS A',
+            '10':                     'CLASS B',
+            '110':                    'CLASS C',
+            '1110':                   'CLASS D MULTICAST',
+            '11100000':               'CLASS D LINKLOCAL',
+            '1111':                   'CLASS E',
+            '00001010':               'PRIVATE RFC1918',    # 10/8
+            '101011000001':           'PRIVATE RFC1918',    # 172.16/12
+            '1100000010101000':       'PRIVATE RFC1918',    # 192.168/16
+        },
         6: {
-            '00000000'              : 'RESERVED',       # ::/8
-            '00000001'              : 'UNASSIGNED',     # 100::/8
-            '0000001'               : 'NSAP',           # 200::/7
-            '0000010'               : 'IPX',            # 400::/7
-            '0000011'               : 'UNASSIGNED',     # 600::/7
-            '00001'                 : 'UNASSIGNED',     # 800::/5
-            '0001'                  : 'UNASSIGNED',     # 1000::/4
-            '0010000000000000'      : 'RESERVED',       # 2000::/16 Reserved
-            '0010000000000001'      : 'ASSIGNABLE',     # 2001::/16 Sub-TLA Assignments [RFC2450]
-            '00100000000000010000000': 'ASSIGNABLE IANA',  # 2001:0000::/29 - 2001:01F8::/29 IANA
-            '00100000000000010000001': 'ASSIGNABLE APNIC', # 2001:0200::/29 - 2001:03F8::/29 APNIC
-            '00100000000000010000010': 'ASSIGNABLE ARIN',  # 2001:0400::/29 - 2001:05F8::/29 ARIN
-            '00100000000000010000011': 'ASSIGNABLE RIPE',  # 2001:0600::/29 - 2001:07F8::/29 RIPE NCC
-            '0010000000000010'      : '6TO4',           # 2002::/16 "6to4" [RFC3056]
-            '0011111111111110'      : '6BONE TEST',     # 3ffe::/16 6bone Testing [RFC2471]
-            '0011111111111111'      : 'RESERVED',  # 3fff::/16 Reserved
-            '010'                   : 'GLOBAL-UNICAST', # 4000::/3
-            '011'                   : 'UNASSIGNED',     # 6000::/3
-            '100'                   : 'GEO-UNICAST',    # 8000::/3
-            '101'                   : 'UNASSIGNED',     # a000::/3
-            '110'                   : 'UNASSIGNED',     # c000::/3
-            '1110'                  : 'UNASSIGNED',     # e000::/4
-            '11110'                 : 'UNASSIGNED',     # f000::/5
-            '111110'                : 'UNASSIGNED',     # f800::/6
-            '1111110'               : 'UNASSIGNED',     # fc00::/7
-            '111111100'             : 'UNASSIGNED',     # fe00::/9
-            '1111111010'            : 'LINKLOCAL',      # fe80::/10
-            '1111111011'            : 'SITELOCAL',      # fec0::/10
-            '11111111'              : 'MULTICAST',      # ff00::/8
-            '0' * 96                : 'IPV4COMP',       # ::/96
-            '0' * 80 + '1' * 16     : 'IPV4MAP',        # ::ffff:0:0/96
-            '0' * 128               : 'UNSPECIFIED',    # ::/128
-            '0' * 127 + '1'         : 'LOOPBACK'        # ::1/128
-            }
+            '00000000':               'RESERVED',           # ::/8
+            '00000001':               'UNASSIGNED',         # 100::/8
+            '0000001':                'NSAP',               # 200::/7
+            '0000010':                'IPX',                # 400::/7
+            '0000011':                'UNASSIGNED',         # 600::/7
+            '00001':                  'UNASSIGNED',         # 800::/5
+            '0001':                   'UNASSIGNED',         # 1000::/4
+            '0010000000000000':       'RESERVED',           # 2000::/16 Reserved
+            '0010000000000001':       'ASSIGNABLE',         # 2001::/16 Sub-TLA Assignments [RFC2450]
+            '00100000000000010000000': 'ASSIGNABLE IANA',   # 2001:0000::/29 - 2001:01F8::/29 IANA
+            '00100000000000010000001': 'ASSIGNABLE APNIC',  # 2001:0200::/29 - 2001:03F8::/29 APNIC
+            '00100000000000010000010': 'ASSIGNABLE ARIN',   # 2001:0400::/29 - 2001:05F8::/29 ARIN
+            '00100000000000010000011': 'ASSIGNABLE RIPE',   # 2001:0600::/29 - 2001:07F8::/29 RIPE NCC
+            '0010000000000010':       '6TO4',               # 2002::/16 "6to4" [RFC3056]
+            '0011111111111110':       '6BONE TEST',         # 3ffe::/16 6bone Testing [RFC2471]
+            '0011111111111111':       'RESERVED',           # 3fff::/16 Reserved
+            '010':                    'GLOBAL-UNICAST',     # 4000::/3
+            '011':                    'UNASSIGNED',         # 6000::/3
+            '100':                    'GEO-UNICAST',        # 8000::/3
+            '101':                    'UNASSIGNED',         # a000::/3
+            '110':                    'UNASSIGNED',         # c000::/3
+            '1110':                   'UNASSIGNED',         # e000::/4
+            '11110':                  'UNASSIGNED',         # f000::/5
+            '111110':                 'UNASSIGNED',         # f800::/6
+            '1111110':                'UNASSIGNED',         # fc00::/7
+            '111111100':              'UNASSIGNED',         # fe00::/9
+            '1111111010':             'LINKLOCAL',          # fe80::/10
+            '1111111011':             'SITELOCAL',          # fec0::/10
+            '11111111':               'MULTICAST',          # ff00::/8
+            '0' * 96:                 'IPV4COMP',           # ::/96
+            '0' * 80 + '1' * 16:      'IPV4MAP',            # ::ffff:0:0/96
+            '0' * 128:                'UNSPECIFIED',        # ::/128
+            '0' * 127 + '1':          'LOOPBACK'            # ::1/128
         }
+    }
 
     def __init__(self, ip, mask=None, version=0):
         self.mask = mask
         self.v = 0
         # Parse input
         if ip is None:
-            raise ValueError, "Can not pass None"
+            raise ValueError('Can not pass None')
         elif isinstance(ip, IP):
             self.ip = ip.ip
             self.dq = ip.dq
@@ -177,19 +176,19 @@ class IP(object):
             limit = [32, 128][':' in self.mask]
             inverted = ~self._dqtoi(self.mask)
             count = 0
-            while inverted & 2**count:
+            while inverted & pow(2, count):
                 count += 1
             self.mask = (limit - count)
         else:
-            raise ValueError, "Invalid netmask"
+            raise ValueError('Invalid netmask')
         # Validate subnet size
         if self.v == 6:
             self.dq = self._itodq(self.ip)
             if not 0 <= self.mask <= 128:
-                raise ValueError, "IPv6 subnet size must be between 0 and 128"
+                raise ValueError('IPv6 subnet size must be between 0 and 128')
         elif self.v == 4:
             if not 0 <= self.mask <= 32:
-                raise ValueError, "IPv4 subnet size must be between 0 and 32"
+                raise ValueError('IPv4 subnet size must be between 0 and 32')
 
     def bin(self):
         '''
@@ -236,9 +235,9 @@ class IP(object):
         CLASS A
         '''
         b = self.bin()
-        l = self.v == 4 and 32 or 128
+        self.v == 4 and 32 or 128
         for i in range(len(b), 0, -1):
-            if self._range[self.v].has_key(b[:i]):
+            if b[:i] in self._range[self.v]:
                 return self._range[self.v][b[:i]]
         return 'UNKNOWN'
 
@@ -250,7 +249,7 @@ class IP(object):
         if dq.startswith('0x'):
             ip = long(dq[2:], 16)
             if ip > 0xffffffffffffffffffffffffffffffffL:
-                raise ValueError, "%r: IP address is bigger than 2^128" % dq
+                raise ValueError('%s: IP address is bigger than 2^128' % dq)
             if ip <= 0xffffffff:
                 self.v = 4
             else:
@@ -259,35 +258,40 @@ class IP(object):
 
         # IPv6
         if ':' in dq:
-            hx = dq.split(':') # split hextets
+            # Split hextets
+            hx = dq.split(':')
             if ':::' in dq:
-                raise ValueError, "%r: IPv6 address can't contain :::" % dq
+                raise ValueError("%s: IPv6 address can't contain :::" % dq)
             # Mixed address (or 4-in-6), ::ffff:192.0.2.42
             if '.' in dq:
                 return self._dqtoi(hx[-1])
             if len(hx) > 8:
-                raise ValueError, "%r: IPv6 address with more than 8 hexletts" % dq
+                raise ValueError('%s: IPv6 address with more than 8 hexlets' % dq)
             elif len(hx) < 8:
                 # No :: in address
                 if not '' in hx:
-                    raise ValueError, "%r: IPv6 address invalid: compressed format malformed" % dq
+                    raise ValueError('%s: IPv6 address invalid: ' +
+                        'compressed format malformed' % dq)
                 elif not (dq.startswith('::') or dq.endswith('::')) and len([x for x in hx if x == '']) > 1:
-                    raise ValueError, "%r: IPv6 address invalid: compressed format malformed" % dq
+                    raise ValueError('%s: IPv6 address invalid: ' +
+                        'compressed format malformed' % dq)
                 ix = hx.index('')
-                px = len(hx[ix+1:])
-                for x in xrange(ix+px+1, 8):
+                px = len(hx[ix + 1:])
+                for x in xrange(ix + px + 1, 8):
                     hx.insert(ix, '0')
             elif dq.endswith('::'):
                 pass
             elif '' in hx:
-                raise ValueError, "%r: IPv6 address invalid: compressed format detected in full notation" % dq
+                raise ValueError('%s: IPv6 address invalid: ' +
+                    'compressed format detected in full notation' % dq())
             ip = ''
             hx = [x == '' and '0' or x for x in hx]
             for h in hx:
                 if len(h) < 4:
                     h = '%04x' % int(h, 16)
                 if not 0 <= int(h, 16) <= 0xffff:
-                    raise ValueError, "%r: IPv6 address invalid: hextets should be between 0x0000 and 0xffff" % dq
+                    raise ValueError('%r: IPv6 address invalid: ' +
+                        'hexlets should be between 0x0000 and 0xffff' % dq)
                 ip += h
             self.v = 6
             return long(ip, 16)
@@ -301,26 +305,33 @@ class IP(object):
             q = dq.split('.')
             q.reverse()
             if len(q) > 4:
-                raise ValueError, "%r: IPv4 address invalid: more than 4 bytes" % dq
+                raise ValueError('%s: IPv4 address invalid: ' +
+                    'more than 4 bytes' % dq)
             for x in q:
                 if not 0 <= int(x) <= 255:
-                    raise ValueError, "%r: IPv4 address invalid: bytes should be between 0 and 255" % dq
+                    raise ValueError('%s: IPv4 address invalid: ' +
+                        'bytes should be between 0 and 255' % dq)
             while len(q) < 4:
                 q.insert(1, '0')
             self.v = 4
             return sum(long(byte) << 8 * index for index, byte in enumerate(q))
 
-        raise ValueError, "Invalid address input"
+        raise ValueError('Invalid address input')
 
     def _itodq(self, n):
         '''
         Convert long to dotquad or hextet.
         '''
         if self.v == 4:
-            return '.'.join(map(str, [(n>>24) & 0xff, (n>>16) & 0xff, (n>>8) & 0xff, n & 0xff]))
+            return '.'.join(map(str, [
+                (n >> 24) & 0xff,
+                (n >> 16) & 0xff,
+                (n >> 8) & 0xff,
+                n & 0xff,
+            ]))
         else:
             n = '%032x' % n
-            return ':'.join(n[4*x:4*x+4] for x in xrange(0, 8))
+            return ':'.join(n[4 * x:4 * x + 4] for x in xrange(0, 8))
 
     def __str__(self):
         '''
@@ -381,9 +392,10 @@ class IP(object):
             if self.bin().startswith('0' * 96):
                 return IP(long(self), version=4)
             elif long(self) & 0x20020000000000000000000000000000L:
-                return IP((long(self)-0x20020000000000000000000000000000L)>>80, version=4)
+                return IP((long(self) - 0x20020000000000000000000000000000L) >> 80, version=4)
             else:
-                return ValueError, "%r: IPv6 address is not IPv4 compatible, nor a 6-to-4 IP" % self.dq
+                return ValueError('%s: IPv6 address is not IPv4 compatible, ' +
+                    'nor an 6-to-4 IP' % self.dq)
 
     @classmethod
     def from_bin(cls, value):
@@ -393,7 +405,7 @@ class IP(object):
         elif len(value) == 128:
             return cls(long(value, 2))
         else:
-            return ValueError, "%r: invalid binary notation" % (value,)
+            return ValueError('%r: invalid binary notation' % (value,))
 
     @classmethod
     def from_hex(cls, value):
@@ -402,7 +414,7 @@ class IP(object):
         elif len(value) == 32:
             return cls(long(value, 16))
         else:
-            raise ValueError, "%r: invalid hexadecimal notation" % (value,)
+            raise ValueError('%r: invalid hexadecimal notation' % (value,))
 
     def to_ipv6(self, type='6-to-4'):
         '''
@@ -415,13 +427,23 @@ class IP(object):
         assert type in ['6-to-4', 'compat'], 'Conversion type not supported'
         if self.v == 4:
             if type == '6-to-4':
-                return IP(0x20020000000000000000000000000000L | long(self)<<80, version=6)
+                return IP(0x20020000000000000000000000000000L | long(self) << 80, version=6)
             elif type == 'compat':
                 return IP(long(self), version=6)
         else:
             return self
 
     def to_reverse(self):
+        '''
+        Convert the IP address to a PTR record in .in-addr.arpa for IPv4 and
+        .ip6.arpa for IPv6 addresses.
+
+        >>> ip = IP('192.0.2.42')
+        >>> print ip.to_reverse()
+        42.2.0.192.in-addr.arpa
+        >>> print ip.to_ipv6().to_reverse()
+        0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.a.2.2.0.0.0.0.c.2.0.0.2.ip6.arpa
+        '''
         if self.v == 4:
             return '.'.join(list(self.dq.split('.')[::-1]) + ['in-addr', 'arpa'])
         else:
@@ -439,7 +461,7 @@ class Network(IP):
     Network slice calculations.
 
     :param ip: network address
-    :type ip: :class:`IP` or str or long or int 
+    :type ip: :class:`IP` or str or long or int
     :param mask: netmask
     :type mask: int or str
 
@@ -451,57 +473,95 @@ class Network(IP):
 
     def netmask(self):
         '''
-        Network netmask derived from subnet size.
+        Network netmask derived from subnet size, as IP object.
 
         >>> localnet = Network('127.0.0.1/8')
         >>> print localnet.netmask()
         255.0.0.0
         '''
+        return IP(self.netmask_long(), version=self.version())
+
+    def netmask_long(self):
+        '''
+        Network netmask derived from subnet size, as long.
+
+        >>> localnet = Network('127.0.0.1/8')
+        >>> print localnet.netmask_long()
+        4278190080
+        '''
         if self.version() == 4:
-            return IP((0xffffffffL >> (32-self.mask)) << (32-self.mask), version=self.version())
+            return (0xffffffffL >> (32 - self.mask)) << (32 - self.mask)
         else:
-            return IP((0xffffffffffffffffffffffffffffffffL >> (128-self.mask)) << (128-self.mask), version=self.version())
+            return (0xffffffffffffffffffffffffffffffffL >> (128 - self.mask)) << (128 - self.mask)
 
     def network(self):
         '''
-        Network address.
+        Network address, as IP object.
 
         >>> localnet = Network('127.128.99.3/8')
         >>> print localnet.network()
         127.0.0.0
         '''
-        return IP(self.ip & long(self.netmask()), version=self.version())
+        return IP(self.network_long(), version=self.version())
+
+    def network_long(self):
+        '''
+        Network address, as long.
+
+        >>> localnet = Network('127.128.99.3/8')
+        >>> print localnet.network_long()
+        2130706432
+        '''
+        return self.ip & self.netmask_long()
 
     def broadcast(self):
         '''
-        Broadcast address.
+        Broadcast address, as IP object.
 
         >>> localnet = Network('127.0.0.1/8')
         >>> print localnet.broadcast()
         127.255.255.255
         '''
-        # XXX: IPv6 doesn't have a broadcast address, but it's used for other 
-        #      calculations such as <Network.host_last>.
+        # XXX: IPv6 doesn't have a broadcast address, but it's used for other
+        #      calculations such as <Network.host_last>
+        return IP(self.broadcast_long(), version=self.version())
+
+    def broadcast_long(self):
+        '''
+        Broadcast address, as long.
+
+        >>> localnet = Network('127.0.0.1/8')
+        >>> print localnet.broadcast_long()
+        2147483647
+        '''
         if self.version() == 4:
-            return IP(long(self.network()) | (0xffffffff - long(self.netmask())), version=self.version())
+            return self.network_long() | (0xffffffffL - self.netmask_long())
         else:
-            return IP(long(self.network()) | (0xffffffffffffffffffffffffffffffffL - long(self.netmask())), version=self.version())
+            return self.network_long() \
+                | (0xffffffffffffffffffffffffffffffffL - self.netmask_long())
 
     def host_first(self):
         '''
         First available host in this subnet.
         '''
-        if (self.version() == 4 and self.mask == 32) or (self.version() == 6 and self.mask == 128):
+        if (self.version() == 4 and self.mask > 30) or \
+                (self.version() == 6 and self.mask > 126):
             return self
-        return IP(long(self.network())+1, version=self.version())
+        else:
+            return IP(self.network_long() + 1, version=self.version())
 
     def host_last(self):
         '''
         Last available host in this subnet.
         '''
-        if (self.version() == 4 and self.mask == 32) or (self.version() == 6 and self.mask == 128):
+        if (self.version() == 4 and self.mask == 32) or \
+                (self.version() == 6 and self.mask == 128):
             return self
-        return IP(long(self.broadcast())-1, version=self.version())
+        elif (self.version() == 4 and self.mask == 31) or \
+                (self.version() == 6 and self.mask == 127):
+            return IP(long(self) + 1, version=self.version())
+        else:
+            return IP(self.broadcast_long() - 1, version=self.version())
 
     def in_network(self, other):
         '''
@@ -545,15 +605,16 @@ class Network(IP):
             slice_step = key.step or 1
             arr = list()
             while x < slice_stop:
-                arr.append(IP(long(self)+x))
+                arr.append(IP(long(self) + x))
                 x += slice_step
             return tuple(arr)
         else:
-            return IP(long(self)+key)
+            return IP(long(self) + key)
 
     def __iter__(self):
         '''
-        Generate a range of ip addresses within the network.
+        Generate a range of usable host IP addresses within the network, as IP
+        objects.
 
         >>> for ip in Network('192.168.114.0/30'):
         ...     print str(ip)
@@ -563,10 +624,12 @@ class Network(IP):
         192.168.114.2
         192.168.114.3
         '''
-        x = 0L
-        while x < self.size():
-            yield IP(long(self)+x)
-            x += 1L
+
+        curr = long(self.host_first())
+        stop = long(self.host_last())
+        while curr <= stop:
+            yield IP(curr)
+            curr += 1
 
     def has_key(self, ip):
         '''
@@ -603,7 +666,7 @@ if __name__ == '__main__':
         ('10.10.0.0', '255.255.255.0', ['10.10.0.20', '10.10.10.20']),
         ('2001:dead:beef:1:c01d:c01a::', 'ffff:ffff:ffff::', ['2001:dead:beef:babe::']),
         ('10.10.0.0/255.255.240.0', None, ['10.10.0.20', '10.10.250.0']),
-        ]
+    ]
 
     for ip, mask, test_ip in tests:
         net = Network(ip, mask)
